searchData={"items":[{"type":"module","title":"OCI","doc":"Documentation for `OCI`.","ref":"OCI.html"},{"type":"behaviour","title":"OCI.Auth.Adapter","doc":"Adapter for authenticating requests to the OCI registry.","ref":"OCI.Auth.Adapter.html"},{"type":"callback","title":"OCI.Auth.Adapter.authenticate/2","doc":"","ref":"OCI.Auth.Adapter.html#c:authenticate/2"},{"type":"type","title":"OCI.Auth.Adapter.authorization_t/0","doc":"Represents the authorization header value.\nThis is the full authorization header value, including the scheme and credentials.","ref":"OCI.Auth.Adapter.html#t:authorization_t/0"},{"type":"callback","title":"OCI.Auth.Adapter.authorize/2","doc":"","ref":"OCI.Auth.Adapter.html#c:authorize/2"},{"type":"callback","title":"OCI.Auth.Adapter.challenge/1","doc":"","ref":"OCI.Auth.Adapter.html#c:challenge/1"},{"type":"type","title":"OCI.Auth.Adapter.credentials_enc_t/0","doc":"Represents encoded credentials string.\nFor Basic auth, this is the base64 encoded username:password string.\nFor Bearer auth, this is the encoded token string.","ref":"OCI.Auth.Adapter.html#t:credentials_enc_t/0"},{"type":"type","title":"OCI.Auth.Adapter.credentials_t/0","doc":"Represents decoded credentials string.\nFor Basic auth, this is the raw username:password string.\nFor Bearer auth, this is the decoded token string.","ref":"OCI.Auth.Adapter.html#t:credentials_t/0"},{"type":"type","title":"OCI.Auth.Adapter.error_details_t/0","doc":"","ref":"OCI.Auth.Adapter.html#t:error_details_t/0"},{"type":"callback","title":"OCI.Auth.Adapter.init/1","doc":"","ref":"OCI.Auth.Adapter.html#c:init/1"},{"type":"type","title":"OCI.Auth.Adapter.scheme_t/0","doc":"Represents the authentication scheme type.\nCurrently supports \"Basic\" and \"Bearer\" authentication methods.","ref":"OCI.Auth.Adapter.html#t:scheme_t/0"},{"type":"type","title":"OCI.Auth.Adapter.subject_t/0","doc":"The subject to set on the request context to be used for authorization.\n\nAvailable in: `conn.assigns[:oci_ctx]`","ref":"OCI.Auth.Adapter.html#t:subject_t/0"},{"type":"type","title":"OCI.Auth.Adapter.t/0","doc":"","ref":"OCI.Auth.Adapter.html#t:t/0"},{"type":"module","title":"OCI.Auth.Static","doc":"Static auth adapter for OCI.\n\nThis adapter is used to authenticate requests using a static list of users and their permissions.\n\nUseful for testing and development. You shouldn't use it in production.","ref":"OCI.Auth.Static.html"},{"type":"type","title":"OCI.Auth.Static.t/0","doc":"","ref":"OCI.Auth.Static.html#t:t/0"},{"type":"module","title":"OCI.Auth.Static.User","doc":"","ref":"OCI.Auth.Static.User.html"},{"type":"type","title":"OCI.Auth.Static.User.t/0","doc":"","ref":"OCI.Auth.Static.User.html#t:t/0"},{"type":"module","title":"OCI.Context","doc":"Context for OCI requests.\n\nThis module is responsible for storing the context of an OCI request.","ref":"OCI.Context.html"},{"type":"type","title":"OCI.Context.t/0","doc":"","ref":"OCI.Context.html#t:t/0"},{"type":"module","title":"OCI.Error","doc":"Error codes for the OCI Registry API.\n\nThis module defines the error codes and messages for the OCI Registry API.\nIt also provides a function to initialize an error struct with a given code and details.","ref":"OCI.Error.html"},{"type":"function","title":"OCI.Error.init/2","doc":"","ref":"OCI.Error.html#init/2"},{"type":"type","title":"OCI.Error.t/0","doc":"","ref":"OCI.Error.html#t:t/0"},{"type":"module","title":"OCI.Inspector","doc":"Helper functions for debugging OCI conformance tests.\n\nThis module provides utilities for inspecting and debugging OCI (Open Container Initiative)\nconformance tests. It allows for runtime inspection of request details and process state,\nand provides a way to set up debugging breakpoints during test execution of specific tests so\nyou don't have to breakpoint through hundreds of unrelated requests.","ref":"OCI.Inspector.html"},{"type":"module","title":"Usage - OCI.Inspector","doc":"The module is primarily used in conjunction with OCI conformance tests. When a request\nincludes the `x-oci-conformance-test` header, the inspector will track the request and\nenable debugging capabilities.\n\nAdd a `x-oci-conformance-test` header to the request to enable debugging in the appropriate\nDistribution Spec ([example: 02 Push Test](.tmp/oci-conformance/distribution-spec/conformance/02_push_test.go)):\n\n```go\nreq.SetHeader(\"x-oci-conformance-test\", g.CurrentSpecReport().FullText())\n```\n\nIn elixir, put an `OCI.Inspector.inspect/2` call in your plug pipeline to enable debugging. When a header comes through for inspection,\nthe proccess dictionary will be populated with the request id and the test name to enable granular breakpointing by specific HTTP requests,\nnot just line number.\n\n```elixir\ndef call(conn, %{registry: registry}) do\n  conn =\n    conn\n    |> put_private(:oci_registry, registry)\n    |> authenticate()\n    |> OCI.Inspector.inspect(\"after:authenticate/1\")\nend\n```\n\nPut a debugger at the line of code you want to breakpoint and pass in the `binding()`, the breakpoint will only trigger for the specific HTTP request.\n\n```elixir\ndef verify_digest(data, digest) do\n  OCI.Inspector.pry(binding())\n  # ... code\nend\n```","ref":"OCI.Inspector.html#module-usage"},{"type":"module","title":"Features - OCI.Inspector","doc":"- Request inspection with detailed logging\n- Process state tracking\n- Runtime debugging capabilities\n- Test case identification","ref":"OCI.Inspector.html#module-features"},{"type":"function","title":"OCI.Inspector.inspect/2","doc":"Inspects an incoming request and sets up debugging context if it's a conformance test.","ref":"OCI.Inspector.html#inspect/2"},{"type":"function","title":"Parameters - OCI.Inspector.inspect/2","doc":"- `conn` - The Plug.Conn struct representing the incoming request\n  - `label` - Optional label for the inspection output (default: \"none\")","ref":"OCI.Inspector.html#inspect/2-parameters"},{"type":"function","title":"Returns - OCI.Inspector.inspect/2","doc":"- The unmodified Plug.Conn struct","ref":"OCI.Inspector.html#inspect/2-returns"},{"type":"function","title":"Examples - OCI.Inspector.inspect/2","doc":"iex> conn = Plug.Conn.put_req_header(conn, \"x-oci-conformance-test\", \"test-name\")\n    iex> OCI.Inspector.inspect(conn)\n    %Plug.Conn{...}","ref":"OCI.Inspector.html#inspect/2-examples"},{"type":"function","title":"OCI.Inspector.log_info/3","doc":"","ref":"OCI.Inspector.html#log_info/3"},{"type":"function","title":"OCI.Inspector.pry/1","doc":"Sets up a debugging breakpoint for the current process if it's part of a conformance test.\n\nThis function will only activate the debugger if the process has been marked\nas part of a conformance test by a previous call to `inspect/2`.","ref":"OCI.Inspector.html#pry/1"},{"type":"function","title":"Parameters - OCI.Inspector.pry/1","doc":"- `binding` - The current binding context for the debugger","ref":"OCI.Inspector.html#pry/1-parameters"},{"type":"function","title":"Returns - OCI.Inspector.pry/1","doc":"- `nil` if no inspector context is found\n  - The result of `IEx.pry()` if inspector context exists","ref":"OCI.Inspector.html#pry/1-returns"},{"type":"type","title":"OCI.Inspector.t/0","doc":"Represents the state of an OCI Inspector instance.","ref":"OCI.Inspector.html#t:t/0"},{"type":"module","title":"OCI.Pagination","doc":"Provides pagination functionality for OCI registry operations, particularly for listing tags.\n\nThis module handles the pagination parameters and results for operations that return\nlarge sets of data, such as listing repository tags. It supports the OCI Distribution\nSpecification's pagination model using `n` (number of results) and `last` (last seen value)\nparameters.","ref":"OCI.Pagination.html"},{"type":"type","title":"OCI.Pagination.t/0","doc":"","ref":"OCI.Pagination.html#t:t/0"},{"type":"module","title":"OCI.Plug","doc":"A Plug for handling OCI (Open Container Initiative) requests.","ref":"OCI.Plug.html"},{"type":"function","title":"OCI.Plug.authenticate/1","doc":"","ref":"OCI.Plug.html#authenticate/1"},{"type":"module","title":"OCI.Plug.Context","doc":"Sets up the OCI context for the request.\n\nThis is used to track the subject, endpoint, resource, repo, and method for the request.\n\nThe context is stored in the `conn.assigns[:oci_ctx]` map.","ref":"OCI.Plug.Context.html"},{"type":"function","title":"OCI.Plug.Context.call/2","doc":"Callback implementation for  Plug.call/2 .","ref":"OCI.Plug.Context.html#call/2"},{"type":"function","title":"OCI.Plug.Context.init/1","doc":"Callback implementation for  Plug.init/1 .","ref":"OCI.Plug.Context.html#init/1"},{"type":"module","title":"OCI.Plug.Handler","doc":"Handles OCI requests.\n\nThis module is responsible for dispatching OCI requests to the appropriate\nhandler function.\n\nIt also validates the repository name and handles pagination.","ref":"OCI.Plug.Handler.html"},{"type":"function","title":"OCI.Plug.Handler.handle/1","doc":"","ref":"OCI.Plug.Handler.html#handle/1"},{"type":"module","title":"OCI.Plug.Parser","doc":"Parses the request body for OCI requests.\n\nThis module is responsible for parsing the request body for OCI requests.\nIt implements the `Plug.Parsers` behaviour to handle different content types\nspecific to OCI (Open Container Initiative) operations.","ref":"OCI.Plug.Parser.html"},{"type":"module","title":"Content Types Handled - OCI.Plug.Parser","doc":"- `application/octet-stream`: For binary blob uploads\n- `application/vnd.oci.image.manifest.v1+json`: For OCI image manifests","ref":"OCI.Plug.Parser.html#module-content-types-handled"},{"type":"type","title":"OCI.Plug.Parser.opts/0","doc":"","ref":"OCI.Plug.Parser.html#t:opts/0"},{"type":"function","title":"OCI.Plug.Parser.parse/5","doc":"Parses the request body based on the content type.","ref":"OCI.Plug.Parser.html#parse/5"},{"type":"function","title":"Content Types Handled - OCI.Plug.Parser.parse/5","doc":"","ref":"OCI.Plug.Parser.html#parse/5-content-types-handled"},{"type":"function","title":"application/octet-stream - OCI.Plug.Parser.parse/5","doc":"Handles binary blob uploads by reading the full body and storing\nit in the connection assigns under the `:oci_blob_chunk` key.","ref":"OCI.Plug.Parser.html#parse/5-application-octet-stream"},{"type":"function","title":"application/vnd.oci.image.manifest.v1+json - OCI.Plug.Parser.parse/5","doc":"Handles OCI image manifest uploads by:\n1. Reading the full body\n2. Computing its SHA256 digest\n3. Storing the digest in the connection assigns\n4. Decoding the JSON manifest","ref":"OCI.Plug.Parser.html#parse/5-application-vnd-oci-image-manifest-v1-json"},{"type":"function","title":"Other Content Types - OCI.Plug.Parser.parse/5","doc":"Passes through to the next parser in the chain.","ref":"OCI.Plug.Parser.html#parse/5-other-content-types"},{"type":"function","title":"Parameters - OCI.Plug.Parser.parse/5","doc":"- conn: The Plug.Conn struct\n  - type: The content type\n  - subtype: The content subtype\n  - headers: The request headers\n  - opts: Parser options containing a :json_decoder key for manifest parsing","ref":"OCI.Plug.Parser.html#parse/5-parameters"},{"type":"function","title":"Returns - OCI.Plug.Parser.parse/5","doc":"- For octet-stream: `{:ok, %{}, conn}` on successful parsing\n  - For manifest: `{:ok, manifest, conn}` on successful parsing\n  - For other types: `{:next, conn}` to pass to the next parser\n  - `{:error, reason}` on failure\n  - Raises `Plug.Parsers.ParseError` on JSON decode failure for manifests","ref":"OCI.Plug.Parser.html#parse/5-returns"},{"type":"module","title":"OCI.Registry","doc":"Registry wraps storage and auth adapters and handles common logic for OCI\nlike validating manifests and tags.","ref":"OCI.Registry.html"},{"type":"function","title":"OCI.Registry.adapter/1","doc":"Extracts the module name from a struct.","ref":"OCI.Registry.html#adapter/1"},{"type":"function","title":"Examples - OCI.Registry.adapter/1","doc":"iex> OCI.Registry.adapter(%OCI.Storage.Local{path: \"/tmp\"})\n    OCI.Storage.Local","ref":"OCI.Registry.html#adapter/1-examples"},{"type":"function","title":"OCI.Registry.api_version/0","doc":"Returns the API version string used in OCI registry paths.\n\nThis is used to construct paths for all registry operations.\nCurrently returns \"v2\" as per the OCI Distribution Specification.","ref":"OCI.Registry.html#api_version/0"},{"type":"function","title":"Examples - OCI.Registry.api_version/0","doc":"iex> OCI.Registry.api_version()\n    \"v2\"","ref":"OCI.Registry.html#api_version/0-examples"},{"type":"function","title":"OCI.Registry.authenticate/2","doc":"","ref":"OCI.Registry.html#authenticate/2"},{"type":"function","title":"OCI.Registry.authorize/2","doc":"","ref":"OCI.Registry.html#authorize/2"},{"type":"function","title":"OCI.Registry.blob_exists?/4","doc":"","ref":"OCI.Registry.html#blob_exists?/4"},{"type":"function","title":"OCI.Registry.blobs_digest_path/2","doc":"Generates the path for a blob with the given digest in a repository.","ref":"OCI.Registry.html#blobs_digest_path/2"},{"type":"function","title":"Parameters - OCI.Registry.blobs_digest_path/2","doc":"- repo: The repository name\n  - digest: The digest of the blob (e.g. \"sha256:abc123...\")","ref":"OCI.Registry.html#blobs_digest_path/2-parameters"},{"type":"function","title":"Returns - OCI.Registry.blobs_digest_path/2","doc":"A string representing the full path to the blob.","ref":"OCI.Registry.html#blobs_digest_path/2-returns"},{"type":"function","title":"Examples - OCI.Registry.blobs_digest_path/2","doc":"iex> OCI.Registry.blobs_digest_path(\"myrepo\", \"sha256:abc123\")\n    \"/v2/myrepo/blobs/sha256:abc123\"","ref":"OCI.Registry.html#blobs_digest_path/2-examples"},{"type":"function","title":"OCI.Registry.blobs_uploads_path/2","doc":"Generates the path for an ongoing blob upload session.","ref":"OCI.Registry.html#blobs_uploads_path/2"},{"type":"function","title":"Parameters - OCI.Registry.blobs_uploads_path/2","doc":"- repo: The repository name\n  - uuid: The unique identifier for the upload session","ref":"OCI.Registry.html#blobs_uploads_path/2-parameters"},{"type":"function","title":"Returns - OCI.Registry.blobs_uploads_path/2","doc":"A string representing the full path to the upload session.","ref":"OCI.Registry.html#blobs_uploads_path/2-returns"},{"type":"function","title":"Examples - OCI.Registry.blobs_uploads_path/2","doc":"iex> OCI.Registry.blobs_uploads_path(\"myrepo\", \"123e4567-e89b-12d3-a456-426614174000\")\n    \"/v2/myrepo/blobs/uploads/123e4567-e89b-12d3-a456-426614174000\"","ref":"OCI.Registry.html#blobs_uploads_path/2-examples"},{"type":"function","title":"OCI.Registry.calculate_range/2","doc":"Calculates the range of a chunk of data.","ref":"OCI.Registry.html#calculate_range/2"},{"type":"function","title":"Examples - OCI.Registry.calculate_range/2","doc":"iex> OCI.Registry.calculate_range(\"hello\", 0)\n  \"0-4\"\n  iex> OCI.Registry.calculate_range(\"hello\", 1)\n  \"1-5\"","ref":"OCI.Registry.html#calculate_range/2-examples"},{"type":"function","title":"OCI.Registry.cancel_blob_upload/4","doc":"","ref":"OCI.Registry.html#cancel_blob_upload/4"},{"type":"function","title":"OCI.Registry.challenge/1","doc":"","ref":"OCI.Registry.html#challenge/1"},{"type":"function","title":"OCI.Registry.complete_blob_upload/5","doc":"","ref":"OCI.Registry.html#complete_blob_upload/5"},{"type":"function","title":"OCI.Registry.delete_blob/4","doc":"","ref":"OCI.Registry.html#delete_blob/4"},{"type":"function","title":"OCI.Registry.delete_manifest/4","doc":"","ref":"OCI.Registry.html#delete_manifest/4"},{"type":"function","title":"OCI.Registry.get_blob/4","doc":"","ref":"OCI.Registry.html#get_blob/4"},{"type":"function","title":"OCI.Registry.get_blob_upload_offset/4","doc":"","ref":"OCI.Registry.html#get_blob_upload_offset/4"},{"type":"function","title":"OCI.Registry.get_blob_upload_status/4","doc":"Gets the status of an ongoing blob upload.","ref":"OCI.Registry.html#get_blob_upload_status/4"},{"type":"function","title":"Parameters - OCI.Registry.get_blob_upload_status/4","doc":"- registry: The registry instance\n  - repo: The repository name\n  - uuid: The upload session ID","ref":"OCI.Registry.html#get_blob_upload_status/4-parameters"},{"type":"function","title":"Returns - OCI.Registry.get_blob_upload_status/4","doc":"- `{:ok, range}` where range is the current range of uploaded bytes\n  - `{:error, :BLOB_UPLOAD_UNKNOWN}` if the upload doesn't exist","ref":"OCI.Registry.html#get_blob_upload_status/4-returns"},{"type":"function","title":"OCI.Registry.get_manifest/4","doc":"","ref":"OCI.Registry.html#get_manifest/4"},{"type":"function","title":"OCI.Registry.init/1","doc":"Initializes a new registry instance with the given configuration.","ref":"OCI.Registry.html#init/1"},{"type":"function","title":"OCI.Registry.initiate_blob_upload/3","doc":"Initiates a new blob upload for the given repository.\nReturns {:ok, location} on success or {:error, reason} on failure.\nThe location is the full path where the blob should be uploaded.","ref":"OCI.Registry.html#initiate_blob_upload/3"},{"type":"function","title":"OCI.Registry.list_tags/4","doc":"","ref":"OCI.Registry.html#list_tags/4"},{"type":"function","title":"OCI.Registry.load_from_env/0","doc":"","ref":"OCI.Registry.html#load_from_env/0"},{"type":"function","title":"OCI.Registry.load_from_keywordlist/1","doc":"","ref":"OCI.Registry.html#load_from_keywordlist/1"},{"type":"function","title":"OCI.Registry.manifest_exists?/4","doc":"","ref":"OCI.Registry.html#manifest_exists?/4"},{"type":"function","title":"OCI.Registry.manifests_reference_path/2","doc":"Generates the path for a manifest reference in the OCI registry.\n\nThe path follows the OCI Distribution Specification format:\n`/v2/:repository/manifests/:reference`","ref":"OCI.Registry.html#manifests_reference_path/2"},{"type":"function","title":"Examples - OCI.Registry.manifests_reference_path/2","doc":"iex> OCI.Registry.manifests_reference_path(\"myorg/myrepo\", \"latest\")\n    \"/v2/myorg/myrepo/manifests/latest\"\n\n    iex> OCI.Registry.manifests_reference_path(\"library/alpine\", \"sha256:24dda0a1be6293020e5355d4a09b9a8bb72a8b44c27b0ca8560669b8ed52d3ec\")\n    \"/v2/library/alpine/manifests/sha256:24dda0a1be6293020e5355d4a09b9a8bb72a8b44c27b0ca8560669b8ed52d3ec\"","ref":"OCI.Registry.html#manifests_reference_path/2-examples"},{"type":"function","title":"OCI.Registry.mount_blob/5","doc":"Mounts a blob from one repository to another.\nReturns {:ok, location} on success, {:error, :BLOB_UNKNOWN} if the source blob doesn't exist.","ref":"OCI.Registry.html#mount_blob/5"},{"type":"function","title":"OCI.Registry.parse_range/1","doc":"Parses a Content-Range header value into start and end positions.","ref":"OCI.Registry.html#parse_range/1"},{"type":"function","title":"Parameters - OCI.Registry.parse_range/1","doc":"- range: A string in the format \"start-end\" (e.g. \"0-1023\")","ref":"OCI.Registry.html#parse_range/1-parameters"},{"type":"function","title":"Returns - OCI.Registry.parse_range/1","doc":"A tuple of {start, end} integers","ref":"OCI.Registry.html#parse_range/1-returns"},{"type":"function","title":"Examples - OCI.Registry.parse_range/1","doc":"iex> OCI.Registry.parse_range(\"0-1023\")\n    {0, 1023}\n    iex> OCI.Registry.parse_range(\"1024-2047\")\n    {1024, 2047}","ref":"OCI.Registry.html#parse_range/1-examples"},{"type":"function","title":"OCI.Registry.repo_exists?/3","doc":"","ref":"OCI.Registry.html#repo_exists?/3"},{"type":"function","title":"OCI.Registry.sha256/1","doc":"Calculates the SHA-256 hash of the given data and returns it as a lowercase hexadecimal string.","ref":"OCI.Registry.html#sha256/1"},{"type":"function","title":"Parameters - OCI.Registry.sha256/1","doc":"- data: The binary data to hash","ref":"OCI.Registry.html#sha256/1-parameters"},{"type":"function","title":"Returns - OCI.Registry.sha256/1","doc":"A lowercase hexadecimal string representing the SHA-256 hash.","ref":"OCI.Registry.html#sha256/1-returns"},{"type":"function","title":"Examples - OCI.Registry.sha256/1","doc":"iex> OCI.Registry.sha256(\"hello\")\n    \"2cf24dba5fb0a30e26e83b2ac5b9e29e1b161e5c1fa7425e73043362938b9824\"","ref":"OCI.Registry.html#sha256/1-examples"},{"type":"function","title":"OCI.Registry.store_manifest/6","doc":"","ref":"OCI.Registry.html#store_manifest/6"},{"type":"type","title":"OCI.Registry.t/0","doc":"","ref":"OCI.Registry.html#t:t/0"},{"type":"function","title":"OCI.Registry.upload_blob_chunk/6","doc":"Uploads a chunk of data to an existing blob upload.\n\n**Note:** The Content-Range header is not always present in chunk uploads:\n- For PATCH requests, Content-Range is required (validated at plug level)\n- For POST (initial) and PUT (final) requests, Content-Range is optional\n- For monolithic uploads, Content-Range may be omitted entirely\n\nThe `maybe_chunk_range` parameter reflects this variability in the protocol.\nWe cannot assume or calculate the range ourselves because:\n1. Previous chunks may have been uploaded out of order\n2. The client may be using a different upload strategy\n3. The range is only meaningful when provided by the client","ref":"OCI.Registry.html#upload_blob_chunk/6"},{"type":"function","title":"Parameters - OCI.Registry.upload_blob_chunk/6","doc":"- registry: The registry instance\n  - repo: The repository name\n  - uuid: The upload session ID\n  - chunk: The binary data chunk to upload\n  - maybe_chunk_range: Optional Content-Range header value","ref":"OCI.Registry.html#upload_blob_chunk/6-parameters"},{"type":"function","title":"Returns - OCI.Registry.upload_blob_chunk/6","doc":"- `{:ok, location, range}` where location is the URL for the next chunk upload and range is the current range of uploaded bytes\n  - `{:error, reason}` if the upload fails","ref":"OCI.Registry.html#upload_blob_chunk/6-returns"},{"type":"function","title":"OCI.Registry.validate_repository_name/2","doc":"","ref":"OCI.Registry.html#validate_repository_name/2"},{"type":"function","title":"OCI.Registry.verify_digest/2","doc":"Verifies that the given data matches the provided digest.","ref":"OCI.Registry.html#verify_digest/2"},{"type":"function","title":"Parameters - OCI.Registry.verify_digest/2","doc":"- data: The binary data to verify\n  - digest: The digest to verify against (must start with \"sha256:\")","ref":"OCI.Registry.html#verify_digest/2-parameters"},{"type":"function","title":"Returns - OCI.Registry.verify_digest/2","doc":"- `:ok` if the data matches the digest\n  - `{:error, :DIGEST_INVALID}` if the digest is invalid or doesn't match","ref":"OCI.Registry.html#verify_digest/2-returns"},{"type":"function","title":"Examples - OCI.Registry.verify_digest/2","doc":"iex> OCI.Registry.verify_digest(\"hello\", \"sha256:2cf24dba5fb0a30e26e83b2ac5b9e29e1b161e5c1fa7425e73043362938b9824\")\n    :ok\n    iex> OCI.Registry.verify_digest(\"hello\", \"sha256:wronghash\")\n    {:error, :DIGEST_INVALID, %{digest: \"sha256:wronghash\", computed: \"2cf24dba5fb0a30e26e83b2ac5b9e29e1b161e5c1fa7425e73043362938b9824\", msg: \"Digest mismatch\"}}\n    iex> OCI.Registry.verify_digest(\"hello\", \"invalid-digest\")\n    {:error, :DIGEST_INVALID, %{digest: \"invalid-digest\", msg: \"Invalid digest format\"}}","ref":"OCI.Registry.html#verify_digest/2-examples"},{"type":"function","title":"OCI.Registry.verify_upload_order/2","doc":"Verifies that a chunk upload is in the correct order.\n\nWhen no range is provided (nil), the upload is considered valid. This is used for\ninitial POST requests and final PUT requests where ranges are not required.","ref":"OCI.Registry.html#verify_upload_order/2"},{"type":"function","title":"Content-Range header requirements for chunk uploads: - OCI.Registry.verify_upload_order/2","doc":"- Required for PATCH requests (validated at plug level)\n- Must be inclusive on both ends (e.g. \"0-1023\")\n- First chunk must begin with 0\n- Chunks must be uploaded in order\n- Not required for initial POST or final PUT requests\n\n**Note:** While nil ranges are valid for POST/PUT, this is a potential security concern\n    as it could allow empty chunk uploads. This is handled by requiring Content-Range\n    for PATCH requests at the plug level, preventing empty chunk uploads before they\n    reach this verification step.","ref":"OCI.Registry.html#verify_upload_order/2-content-range-header-requirements-for-chunk-uploads"},{"type":"function","title":"Parameters - OCI.Registry.verify_upload_order/2","doc":"- current_size: The current size of uploaded data in bytes\n  - range: The Content-Range header value or nil","ref":"OCI.Registry.html#verify_upload_order/2-parameters"},{"type":"function","title":"Returns - OCI.Registry.verify_upload_order/2","doc":"- `:ok` if the upload is valid\n  - `{:error, :EXT_BLOB_UPLOAD_OUT_OF_ORDER}` if the chunk is out of order","ref":"OCI.Registry.html#verify_upload_order/2-returns"},{"type":"function","title":"Examples - OCI.Registry.verify_upload_order/2","doc":"iex> OCI.Registry.verify_upload_order(0, nil)\n    :ok\n    iex> OCI.Registry.verify_upload_order(1024, \"1024-2047\")\n    :ok\n    iex> OCI.Registry.verify_upload_order(1024, \"2048-3071\")\n    {:error, :EXT_BLOB_UPLOAD_OUT_OF_ORDER, %{current_size: 1024, range: \"2048-3071\"}}","ref":"OCI.Registry.html#verify_upload_order/2-examples"},{"type":"behaviour","title":"OCI.Storage.Adapter","doc":"Behaviour for OCI registry storage backends.\n\nThis module defines the interface that storage adapters must implement to work with OCI (Open Container Initiative) registries.\nA storage adapter is responsible for storing and retrieving container images and their components.","ref":"OCI.Storage.Adapter.html"},{"type":"behaviour","title":"Key Concepts - OCI.Storage.Adapter","doc":"- **Repository**: A collection of related container images (e.g., \"myapp/backend\")\n- **Blob**: Binary data that represents layers of a container image or configuration\n- **Manifest**: A JSON document that describes a container image, including its layers and configuration\n- **Digest**: A unique identifier for a blob, typically a SHA-256 hash\n- **Reference**: A human-readable identifier for a manifest (e.g., \"latest\", \"v1.0.0\")","ref":"OCI.Storage.Adapter.html#module-key-concepts"},{"type":"behaviour","title":"Implementation Guide - OCI.Storage.Adapter","doc":"When implementing this behaviour, you'll need to handle:\n1. Repository initialization and management\n2. Blob storage and retrieval\n3. Manifest storage and retrieval\n4. Tag management\n5. Upload handling for large blobs","ref":"OCI.Storage.Adapter.html#module-implementation-guide"},{"type":"behaviour","title":"Example Usage - OCI.Storage.Adapter","doc":"```elixir\ndefmodule MyStorageAdapter do\n  @behaviour OCI.Storage.Adapter\n\n  defstruct [:path]  # Define your struct fields here\n\n  # Implement all callbacks here\nend\n```","ref":"OCI.Storage.Adapter.html#module-example-usage"},{"type":"callback","title":"OCI.Storage.Adapter.blob_exists?/4","doc":"Checks if a blob exists in the repository and returns its size if found.","ref":"OCI.Storage.Adapter.html#c:blob_exists?/4"},{"type":"callback","title":"OCI.Storage.Adapter.cancel_blob_upload/4","doc":"Cancels an ongoing blob upload session.","ref":"OCI.Storage.Adapter.html#c:cancel_blob_upload/4"},{"type":"callback","title":"OCI.Storage.Adapter.complete_blob_upload/5","doc":"Finalizes a blob upload and verifies the digest.","ref":"OCI.Storage.Adapter.html#c:complete_blob_upload/5"},{"type":"callback","title":"OCI.Storage.Adapter.delete_blob/4","doc":"Deletes a blob from the repository.","ref":"OCI.Storage.Adapter.html#c:delete_blob/4"},{"type":"callback","title":"OCI.Storage.Adapter.delete_manifest/4","doc":"Deletes a manifest from the repository.","ref":"OCI.Storage.Adapter.html#c:delete_manifest/4"},{"type":"type","title":"OCI.Storage.Adapter.error_details_t/0","doc":"","ref":"OCI.Storage.Adapter.html#t:error_details_t/0"},{"type":"callback","title":"OCI.Storage.Adapter.get_blob/4","doc":"Retrieves a blob's content from the repository.","ref":"OCI.Storage.Adapter.html#c:get_blob/4"},{"type":"callback","title":"OCI.Storage.Adapter.get_blob_upload_offset/4","doc":"Gets the total size of an ongoing blob upload.","ref":"OCI.Storage.Adapter.html#c:get_blob_upload_offset/4"},{"type":"callback","title":"OCI.Storage.Adapter.get_blob_upload_status/4","doc":"Gets the status of an ongoing blob upload.","ref":"OCI.Storage.Adapter.html#c:get_blob_upload_status/4"},{"type":"callback","title":"OCI.Storage.Adapter.get_manifest/4","doc":"Retrieves a manifest from the repository.","ref":"OCI.Storage.Adapter.html#c:get_manifest/4"},{"type":"callback","title":"OCI.Storage.Adapter.init/1","doc":"Initializes a new storage adapter instance with the given configuration.","ref":"OCI.Storage.Adapter.html#c:init/1"},{"type":"callback","title":"OCI.Storage.Adapter.initiate_blob_upload/3","doc":"Initiates a blob upload session.","ref":"OCI.Storage.Adapter.html#c:initiate_blob_upload/3"},{"type":"callback","title":"OCI.Storage.Adapter.list_tags/4","doc":"Lists tags in a repository with pagination support.","ref":"OCI.Storage.Adapter.html#c:list_tags/4"},{"type":"callback","title":"OCI.Storage.Adapter.manifest_exists?/4","doc":"Gets metadata about a manifest without retrieving its content.","ref":"OCI.Storage.Adapter.html#c:manifest_exists?/4"},{"type":"callback","title":"OCI.Storage.Adapter.mount_blob/5","doc":"Mounts a blob from one repository to another.","ref":"OCI.Storage.Adapter.html#c:mount_blob/5"},{"type":"callback","title":"OCI.Storage.Adapter.repo_exists?/3","doc":"Checks if a repository exists.","ref":"OCI.Storage.Adapter.html#c:repo_exists?/3"},{"type":"callback","title":"OCI.Storage.Adapter.store_manifest/6","doc":"Stores a manifest in the repository.","ref":"OCI.Storage.Adapter.html#c:store_manifest/6"},{"type":"type","title":"OCI.Storage.Adapter.t/0","doc":"","ref":"OCI.Storage.Adapter.html#t:t/0"},{"type":"callback","title":"OCI.Storage.Adapter.upload_blob_chunk/6","doc":"Uploads a chunk of data to an ongoing blob upload.","ref":"OCI.Storage.Adapter.html#c:upload_blob_chunk/6"},{"type":"callback","title":"OCI.Storage.Adapter.upload_exists?/4","doc":"Checks if an upload exists.","ref":"OCI.Storage.Adapter.html#c:upload_exists?/4"},{"type":"module","title":"OCI.Storage.Local","doc":"Local storage adapter for OCI.\n\nFile system structure:\n```\n /\n├──  /\n│   ├── blobs/\n│   │   └── sha256:   # Stored blobs\n│   ├── uploads/\n│   │   └──  /          # Temporary upload directory\n│   │       └── chunk.*      # Chunked upload files\n│   ├── manifests/           # Manifest storage\n│   │   └── sha256:   # Stored manifests\n│   └── manifest/\n│       └── tags/            # Tag references\n│           └──          # Tag to digest mapping\n```\nThe local storage adapter implements the OCI distribution spec by storing:\n- Blobs in the `blobs/` directory, named by their digest\n- Manifests in the `manifests/` directory, named by their digest\n- Tag references in `manifest/tags/`, mapping tags to manifest digests\n- Temporary uploads in `uploads/ /` during chunked uploads","ref":"OCI.Storage.Local.html"},{"type":"type","title":"OCI.Storage.Local.t/0","doc":"","ref":"OCI.Storage.Local.html#t:t/0"},{"type":"extras","title":"OCI","doc":"# OCI\n\n![OCI Plug Logo](assets/logo.png)\n\n[![Hex.pm](https://img.shields.io/hexpm/v/oci.svg)](https://hex.pm/packages/oci)\n[![Hex.pm](https://img.shields.io/hexpm/dt/oci.svg)](https://hex.pm/packages/oci)\n[![Hex.pm](https://img.shields.io/hexpm/l/oci.svg)](https://hex.pm/packages/oci)\n[![CI](https://github.com/massdriver-cloud/oci/actions/workflows/ci.yml/badge.svg)](https://github.com/massdriver-cloud/oci/actions/workflows/ci.yml)\n[![Credo](https://img.shields.io/badge/Credo-Enabled-brightgreen)](https://github.com/rrrene/credo)\n\nAn [OCI](https://opencontainers.org/) (Open Container Initiative) compliant V2 registry server implementation for Elixir. This library provides a plug-based solution that can be integrated into any Elixir web application, with configurable storage and authentication adapters.\n\n**The included adapters are nowhere near production-ready. Make sure to implement your own before you go and have a bad time.**\n\nThis library is under heavy active development as we extract it from the [Massdriver](https://massdriver.cloud) codebase. `0.0.x` will have many breaking changes.","ref":"readme.html"},{"type":"extras","title":"Features - OCI","doc":"- Full OCI Distribution Specification V2 compliance\n- Pluggable storage backend\n- Configurable authentication\n- Easy integration with Phoenix applications\n- Support for Docker and OCI image formats\n- Compatible with Docker CLI and ORAS tools\n- Support for various repository naming conventions (`nginx`, `hexpm/elixir`, `big-corp/big-team/big-project`)","ref":"readme.html#features"},{"type":"extras","title":"Repository Naming - OCI","doc":"This registry supports the standard OCI repository naming convention with strict `namespace/name`, `org/team/project`, or whatever wild ass `/` party you can dream up.\n\n- ✅ `myapp` - Single-level names\n- ✅ `myorg/myapp` - Standard namespace/name format\n- ✅ `org/team/project` - Multi-level namespaces\n- IT CAN JUST KEEP GOING (I THINK)","ref":"readme.html#repository-naming"},{"type":"extras","title":"Installation - OCI","doc":"The package can be installed by adding `oci` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:oci, \"~> 0.0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - OCI","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Phoenix Integration - OCI","doc":"See an example in [the test setup](./test/support/router.ex).\n\n**config/config.ex**\n\n```elixir\nconfig :oci,\n  max_manifest_size: 4 * 1024 * 1024,\n  max_blob_upload_chunk_size: 10 * 1024 * 1024,\n  enable_blob_deletion: true,\n  enable_manifest_deletion: true,\n  repo_name_pattern: ~r/^([a-z0-9]+(?:[._-][a-z0-9]+)*)(\\/[a-z0-9]+(?:[._-][a-z0-9]+)*)*$/\n  storage: [\n    adapter: OCI.Storage.Local,\n    config: [\n      path: \"./tmp/\"\n    ]\n  ]\n```\n\n**endpoint.ex**\n```elixir\ndefmodule Endpoint do\n  @moduledoc false\n  use Phoenix.Endpoint, otp_app: :oci\n\n  plug(Plug.Parsers,\n    parsers: [OCI.Plug.Parser, :json], # <- stick this bad boi in here. It'll full body read blob uploads and parse/digest manifests.\n    pass: [\"*/*\"],\n    json_decoder: Jason,\n    length: 20_000_000\n  )\n\n  plug(TestRegistryWeb.Router)\nend\n```\n\n**router.ex:**\n\n```elixir\nuse Phoenix.Router\nimport OCI.PhoenixRouter\n\nscope \"/v2\" do\n  forward(\"/\", OCI.Plug, [])\nend\n```","ref":"readme.html#basic-phoenix-integration"},{"type":"extras","title":"ORAS CLI Interaction - OCI","doc":"```bash\n# Push an artifact\noras push localhost:5000/myorg/myapp:latest ./my-artifact.txt\n\n# Pull an artifact\noras pull localhost:5000/myorg/myapp:latest\n```","ref":"readme.html#oras-cli-interaction"},{"type":"extras","title":"Docker CLI Interaction - OCI","doc":"```bash\n# Pull an image\ndocker pull localhost:5000/myorg/myapp:latest\n\n# Push an image\ndocker push localhost:5000/myorg/myapp:latest\n\n# List tags\ncurl -X GET http://localhost:5000/v2/myorg/myapp/tags/list\n```","ref":"readme.html#docker-cli-interaction"},{"type":"extras","title":"Custom Storage Adapter - OCI","doc":"**TODO** See [adapter](./lib/oci/storage/adapter.ex) and [local fs example](./lib/oci/storage/local.ex)","ref":"readme.html#custom-storage-adapter"},{"type":"extras","title":"Custom Authentication - OCI","doc":"**TODO** See [adapter](./lib/oci/auth/adapter.ex) and [basic/static example](./lib/oci/auth/static.ex)","ref":"readme.html#custom-authentication"},{"type":"extras","title":"Development - OCI","doc":"","ref":"readme.html#development"},{"type":"extras","title":"Running Tests - OCI","doc":"This will run basic plug tests and the [conformance suite](./test/support/conformance_suite.ex) against a phoenix endpoint.\n\n```bash\nmix test\n```","ref":"readme.html#running-tests"},{"type":"extras","title":"Contributing - OCI","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Run the QA suite to ensure quality (`mix qa`)\n5. Push to the branch (`git push origin my-new-feature`)\n6. Create new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - OCI","doc":"This project is licensed under the Apache License 2.0 - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"References - OCI","doc":"- [OCI Distribution Specification](https://github.com/opencontainers/distribution-spec)\n- [Docker Registry HTTP API V2](https://docs.docker.com/registry/spec/api/)\n- [ORAS CLI](https://oras.land/cli/)","ref":"readme.html#references"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   Copyright 2024 Massdriver\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}